@@ -1,7 +1,9 @@
 from __future__ import print_function
 import unittest,os,tempfile,sys,glob,subprocess,multiprocessing
+sys.path.append(os.path.dirname(sys.path[0]))
+from pkg_resources import parse_version
 from stat import *
-from testcdo import *
+from cdo import Cdo,CDOException
 import numpy as np
 from matplotlib import pylab as pl
 
@@ -12,10 +14,11 @@ if 'CDF_MOD' in os.environ:
 else:
   CDF_MOD = 'netcdf4'
 
-HOSTNAME = 'luthien'
-DATA_DIR = os.environ.get('HOME')+'/local/data'
+HOSTNAME       = 'luthien'
+DATA_DIR       = os.environ.get('HOME')+'/local/data'
 
-SHOW = 'SHOW' in os.environ
+SHOW           = 'SHOW' in os.environ
+DEBUG          = 'DEBUG' in os.environ
 
 MAINTAINERMODE = 'MAINTAINERMODE' in os.environ
 
@@ -48,7 +51,7 @@ class CdoTest(unittest.TestCase):
 
     def testVersions(self):
         cdo = Cdo()
-        self.assertEqual('1.3.0',cdo.__version__())
+        self.assertEqual('1.3.3',cdo.__version__())
         self.assertTrue(parse_version('1.7.0') <= parse_version(cdo.version()))
 
     def testCDO(self):
@@ -60,11 +63,16 @@ class CdoTest(unittest.TestCase):
             cdo.setCdo('cdo')
 
     def testDbg(self):
+        if not 'DEBUG' in os.environ:
+          cdo = Cdo(cdfMod=CDF_MOD)
+          self.assertEqual(False,cdo.debug)
+          cdo.debug = True
+          self.assertEqual(True,cdo.debug)
+          cdo.debug = False
+
+    def test_V(self):
         cdo = Cdo(cdfMod=CDF_MOD)
-        self.assertEqual(False,cdo.debug)
-        cdo.debug = True
-        self.assertEqual(True,cdo.debug)
-        cdo.debug = False
+        print(cdo.version(verbose=True))
 
     def testOps(self):
         cdo = Cdo(cdfMod=CDF_MOD)
@@ -91,10 +99,10 @@ class CdoTest(unittest.TestCase):
 
     def test_simple(self):
         cdo = Cdo(cdfMod=CDF_MOD)
-        cdo.debug = True
+        cdo.debug = DEBUG
         s   = cdo.sinfov(input="-topo",options="-f nc")
         s   = cdo.sinfov(input="-remapnn,r36x18 -topo",options="-f nc")
-        f   = 'ofile.nc'
+        f   = 'test_ofile.nc'
         cdo.expr("'z=log(abs(topo+1))*9.81'",input="-topo", output=f, options="-f nc")
         s   = cdo.infov(input=f)
         cdo.stdatm("0",output=f,options="-f nc")
@@ -111,9 +119,29 @@ class CdoTest(unittest.TestCase):
         self.assertEqual(["1013.25", "288"],values)
         values = cdo.outputkey("value",input="-stdatm,0,10000")[1::]
         self.assertEqual(["1013.25", "271.913", "288", "240.591"],values)
-        values = cdo.outputkey("level",input="-stdatm,0,10000")[1::]
+        values = cdo.outputkey("lev",input="-stdatm,0,10000")[1::]
         self.assertEqual(["0", "10000","0", "10000"],values)
 
+        # test autoSplit usage
+        levels = cdo.showlevel(input="-stdatm,0,10,20",autoSplit=' ')
+        self.assertEqual([['0','10','20'],['0','10','20']],levels)
+
+        timesExpected = ['2001-01-01T12:00:00',
+          '2001-01-01T13:00:00',
+          '2001-01-01T14:00:00',
+          '2001-01-01T15:00:00',
+          '2001-01-01T16:00:00',
+          '2001-01-01T17:00:00',
+          '2001-01-01T18:00:00',
+          '2001-01-01T19:00:00',
+          '2001-01-01T20:00:00',
+          '2001-01-01T21:00:00']
+        self.assertEqual(timesExpected,
+                         cdo.showtimestamp(input="-settaxis,2001-01-01,12:00,1hour -for,1,10", autoSplit='  '))
+
+        self.assertEqual(['P T'],cdo.showname(input="-stdatm,0"))
+        self.assertEqual(['P','T'],cdo.showname(input="-stdatm,0",autoSplit=' '))
+
     def test_bndLevels(self):
         cdo = Cdo(cdfMod=CDF_MOD)
         ofile = cdo.stdatm(25,100,250,500,875,1400,2100,3000,4000,5000,options = "-f nc")
@@ -124,7 +152,7 @@ class CdoTest(unittest.TestCase):
 
     def test_CDO_options(self):
         cdo = Cdo(cdfMod=CDF_MOD)
-        cdo.debug = True
+        cdo.debug = DEBUG
         names = cdo.showname(input = "-stdatm,0",options = "-f nc")
         self.assertEqual(["P T"],names)
         if cdo.hasLib("sz"):
@@ -138,7 +166,7 @@ class CdoTest(unittest.TestCase):
 
     def test_diff(self):
         cdo = Cdo(cdfMod=CDF_MOD)
-        cdo.debug = True
+        cdo.debug = DEBUG
         diffv = cdo.diffn(input = "-random,r1x1 -random,r1x1")
         print(diffv)
         self.assertEqual(diffv[1].split(' ')[-1],"random")
@@ -159,7 +187,7 @@ class CdoTest(unittest.TestCase):
         press = cdo.stdatm("0",output=ofile,options="-f nc")
         self.assertEqual(ofile,press)
         cdo.setReturnArray()
-        outfile = 'test.nc'
+        outfile = 'test_returnCdf.nc'
         cdf = cdo.stdatm("0",output=outfile,options="-f nc")
         press = cdf.variables["P"][:]
         self.assertEqual(1013.25,press.min())
@@ -178,7 +206,7 @@ class CdoTest(unittest.TestCase):
 
     def test_forceOutput(self):
         cdo = Cdo(cdfMod=CDF_MOD)
-        cdo.debug = True
+        cdo.debug = DEBUG
         outs = []
         # tempfiles
         outs.append(cdo.stdatm("0,10,20"))
@@ -225,13 +253,13 @@ class CdoTest(unittest.TestCase):
 
     def test_combine(self):
         cdo = Cdo(cdfMod=CDF_MOD)
-        cdo.debug = True
+        cdo.debug = DEBUG
         stdatm  = cdo.stdatm("0",options = "-f nc")
         stdatm_ = cdo.stdatm("0",options = "-f nc")
-        print(cdo.diff(input=stdatm + " " + stdatm_))
-        sum = cdo.fldsum(input = stdatm)
-        sum = cdo.fldsum(input = cdo.stdatm("0",options="-f nc"),returnCdf=True)
-        self.assertEqual(288.0,sum.variables["T"][:])
+       #print(cdo.diff(input=stdatm + " " + stdatm_))
+       #sum = cdo.fldsum(input = stdatm)
+       #sum = cdo.fldsum(input = cdo.stdatm("0",options="-f nc"),returnCdf=True)
+       #self.assertEqual(288.0,sum.variables["T"][:])
 
     def test_cdf(self):
         cdo = Cdo(cdfMod=CDF_MOD)
@@ -272,7 +300,7 @@ class CdoTest(unittest.TestCase):
 
     def test_returnArray(self):
         cdo = Cdo(cdfMod=CDF_MOD)
-        cdo.debug = True
+        cdo.debug = DEBUG
         self.assertEqual(False, cdo.stdatm(0,options = '-f nc', returnArray = 'TT'))
 #TODO       temperature = cdo.stdatm(0,options = '-f nc', returnArray = 'T')
 #TODO       self.assertEqual(288.0,temperature.flatten()[0])
@@ -281,7 +309,7 @@ class CdoTest(unittest.TestCase):
 
     def test_returnMaArray(self):
         cdo = Cdo(cdfMod=CDF_MOD)
-        cdo.debug = True
+        cdo.debug = DEBUG
         topo = cdo.topo(options='-f nc',returnMaArray='topo')
         self.assertEqual(-1890.0,round(topo.mean()))
         bathy = cdo.setrtomiss(0,10000,
@@ -302,8 +330,42 @@ class CdoTest(unittest.TestCase):
         self.assertEqual(False,withMask.mask[1,1])
         self.assertEqual(True,withMask.mask[0,1])
 
+#   def test_XrDataset(self):
+#       cdo = Cdo(cdfMod=CDF_MOD)
+#       self.assertTrue(hasattr(cdo, "cdf"))# not in cdo.__dict__)
+#       cdo.setReturnArray()
+#       self.assertTrue(hasattr(cdo, "cdf"))#"cdf" in cdo.__dict__)
+#       cdo.setReturnArray(False)
+#       sum = cdo.fldsum(input = cdo.stdatm("0",options="-f nc"),returnXrDataset=True)
+#       self.assertEqual(1013.25,sum.variables["P"][:])
+#       cdo.unsetReturnArray()
+
+#   def test_returnXrArray(self):
+#       cdo = Cdo(cdfMod=CDF_MOD)
+#       cdo.debug = DEBUG
+#       topo = cdo.topo(options='-f nc',returnXrArray='topo')
+#       self.assertEqual(-1890.0,round(topo.mean()))
+#       bathy = cdo.setrtomiss(0,10000,
+#           input = cdo.topo(options='-f nc'),returnXrArray='topo')
+#       self.assertEqual(-3386.0,round(bathy.mean()))
+#       oro = cdo.setrtomiss(-10000,0,
+#           input = cdo.topo(options='-f nc'),returnXrArray='topo')
+#       self.assertEqual(1142.0,round(oro.mean()))
+#       bathy = cdo.remapnn('r2x2',input = cdo.topo(options = '-f nc'), returnXrArray = 'topo')
+#       self.assertEqual(-4298.0,bathy[0,0])
+#       self.assertEqual(-2669.0,bathy[0,1])
+#       ta = cdo.remapnn('r2x2',input = cdo.topo(options = '-f nc'))
+#       tb = cdo.subc(-2669.0,input = ta)
+#       withMask = cdo.div(input=ta+" "+tb,returnXrArray='topo')
+#       self.assertEqual('--',withMask[0,1].__str__())
+#       self.assertEqual(False,withMask.mask[0,0])
+#       self.assertEqual(False,withMask.mask[1,0])
+#       self.assertEqual(False,withMask.mask[1,1])
+#       self.assertEqual(True,withMask.mask[0,1])
+
     def test_errorException(self):
         cdo = Cdo(cdfMod=CDF_MOD)
+        cdo.__print__('test_errorException')
         self.assertFalse(hasattr(cdo, 'nonExistingMethod'))
         self.failUnlessRaises(CDOException, cdo.max)
         try:
@@ -322,7 +384,7 @@ class CdoTest(unittest.TestCase):
 
     def test_inputArray(self):
         cdo = Cdo(cdfMod=CDF_MOD)
-        cdo.debug = 'DEBUG' in os.environ
+        cdo.debug = DEBUG
         # check for file input
         fileA = cdo.stdatm(0,output='A')
         fileB = cdo.stdatm(0,output='B')
@@ -335,6 +397,37 @@ class CdoTest(unittest.TestCase):
         self.assertEqual([],cdo.diffv(input = ["-stdatm,0",fileB]))
         rm([fileA, fileB])
 
+    def test_splitOps(self):
+        cdo = Cdo(cdfMod=CDF_MOD)
+        cdo.debug = DEBUG
+        pattern = 'stdAtm'
+        cdo.__print__('test_splitOps')
+        resultsFiles = cdo.splitname(input = '-stdatm,0',output = pattern)
+        self.assertTrue(2 <= len(resultsFiles))
+        print(resultsFiles)
+        for var in ['T','P']:
+          print(pattern+var+'.grb')
+          self.assertTrue(pattern+var+'.grb' in resultsFiles)
+        rm(resultsFiles)
+
+        pattern = 'sel'
+        resultsFiles = cdo.splitsel(1,input = '-for,0,9',output = pattern)
+        print(resultsFiles)
+        self.assertTrue(10 <= len(resultsFiles))
+        rm(resultsFiles)
+        for var in range(0,10):
+          self.assertTrue(pattern+'00000'+str(var)+'.grb' in resultsFiles)
+        rm(resultsFiles)
+
+        pattern = 'lev'
+        resultsFiles = cdo.splitlevel(input = '-stdatm,100,2000,5000',output = pattern)
+        self.assertTrue(3 <= len(resultsFiles))
+        print(resultsFiles)
+        rm(resultsFiles)
+        for var in ['0100','2000','5000']:
+          self.assertTrue(pattern+'00'+str(var)+'.grb' in resultsFiles)
+        rm(resultsFiles)
+
     def test_output_set_to_none(self):
         cdo = Cdo(cdfMod=CDF_MOD)
         self.assertEqual(str,type(cdo.topo(output = None)))
@@ -342,11 +435,10 @@ class CdoTest(unittest.TestCase):
 
     def test_libs(self):
         cdo = Cdo(cdfMod=CDF_MOD)
-        cdo.debug = True
+        cdo.debug = DEBUG
         print(cdo.libs)
         self.assertTrue(cdo.hasLib("cdi"),"CDI support missing")
         self.assertTrue(cdo.hasLib("extra"),"netcdf4 support missing")
-        self.assertTrue(cdo.hasLib("sz"),"netcdf4 support missing")
         self.assertTrue(cdo.hasLib("nc4"),"netcdf4 support missing")
         self.assertTrue(cdo.hasLib("netcdf"),"netcdf support missing")
         self.assertTrue(cdo.hasLib("udunits2"),"netcdf support missing")
@@ -382,7 +474,7 @@ class CdoTest(unittest.TestCase):
 
     def test_env(self):
         # clean up
-        tag = '__env_test'
+        tag = 'test___env_test'
         files = glob.glob(tag+'*')
         rm(files)
         files = glob.glob(tag+'*')
@@ -390,34 +482,41 @@ class CdoTest(unittest.TestCase):
 
         # setup
         cdo = Cdo(cdfMod=CDF_MOD)
-        cdo.debug = 'DEBUG' in os.environ
+        cdo.debug = DEBUG
 
+        cdo.__print__('test_env')
         # cdf default
         ifile = cdo.stdatm(10,20,50,100,options='-f nc')
         cdo.splitname(input=ifile,output=tag)
         files = glob.glob(tag+'*')
         files.sort()
-        self.assertEqual(['__env_testP.nc', '__env_testT.nc'],files)
+        self.assertEqual(['%sP.nc'%(tag), '%sT.nc'%(tag)],files)
         rm(files)
 
         # manual setup to nc2 via operator call
-        cdo.splitname(input=ifile,output=tag,env={"CDO_FILE_SUFFIX": ".bar"})
+        cdo.splitname(input=ifile,output=tag,env={"CDO_FILE_SUFFIX": ".foo"})
+        cdo.env = {}
         files = glob.glob(tag+'*')
         files.sort()
-        self.assertEqual(['__env_testP.bar', '__env_testT.bar'],files)
+        self.assertEqual(['%sP.foo'%(tag), '%sT.foo'%(tag)],files)
         rm(files)
 
         # manual setup to nc2 via object setup
-        cdo.env = {"CDO_FILE_SUFFIX": ".foo"}
+        cdo.__print__('test_env:VOR BLA')
+        cdo.env = {"CDO_FILE_SUFFIX": ".bla"}
         cdo.splitname(input=ifile,output=tag)
+        cdo.splitname(input=ifile,output='bla')
+        cdo.__print__('test_env:NACH BLA')
+        cdo.env = {}
         files = glob.glob(tag+'*')
         files.sort()
-        self.assertEqual(['__env_testP.foo', '__env_testT.foo'],files)
+        self.assertEqual(['%sP.bla'%(tag), '%sT.bla'%(tag)],files)
         rm(files)
+        cdo.__print__('test_env:ENDE')
 
     def test_showMaArray(self):
         cdo = Cdo(cdfMod=CDF_MOD)
-        cdo.debug = True
+        cdo.debug = DEBUG
         print(cdo)
         bathy = cdo.setrtomiss(0,10000,
             input = cdo.topo(options='-f nc'),returnMaArray='topo')
@@ -439,13 +538,12 @@ class CdoTest(unittest.TestCase):
         else:
           self.assertEqual('netcdf4',cdo.cdfMod)
 
-
     def test_fillmiss(self):
         cdo = Cdo(cdfMod='netcdf4')
         if 'CDO' in os.environ:
           cdo.setCdo(os.environ.get('CDO'))
 
-        cdo.debug = True
+        cdo.debug = DEBUG
         rand = cdo.setname('v',input = '-random,r25x25 ', options = ' -f nc',output = '/tmp/rand.nc')
         cdf  = cdo.openCdf(rand)
         var  = cdf.variables['v']
@@ -460,13 +558,13 @@ class CdoTest(unittest.TestCase):
         cdf.close()
 
         missRange = '0.25,0.85'
-        withMissRange = 'withMissRange.nc'
+        withMissRange = 'test_withMissRange.nc'
         arOrg = cdo.copy(input = rand,returnMaArray = 'v')
         arWmr = cdo.setrtomiss(missRange,input = rand,output = withMissRange,returnMaArray='v')
         arFm  = cdo.fillmiss(            input = withMissRange,returnMaArray = 'v')
-        arFm1s= cdo.fillmiss2(2,         input = withMissRange,returnMaArray = 'v',output='foo.nc')
+        arFm1s= cdo.fillmiss2(2,         input = withMissRange,returnMaArray = 'v',output='test_foo.nc')
         if 'setmisstonn' in cdo.operators:
-          arM2NN= cdo.setmisstonn(         input = withMissRange,returnMaArray = 'v',output='foo.nc')
+          arM2NN= cdo.setmisstonn(         input = withMissRange,returnMaArray = 'v',output='test_foo.nc')
 
         pool = multiprocessing.Pool(8)
         pool.apply_async(plot, (arOrg, ),{"title":'org'      })#ofile='fmOrg')
@@ -529,6 +627,14 @@ class CdoTest(unittest.TestCase):
         cdo.sinfov(input=cmd)
         cdo.showLog()
 
+        cmd = '-fldmean -mul -random,r20x20 -topo,r20x20'
+        print('# logging with a real file, passed as unicode string')
+        cdo = Cdo(cdfMod=CDF_MOD, logging=True, logFile=u'foo.log')
+        cdo.topo()
+        cdo.temp()
+        cdo.sinfov(input=cmd)
+        cdo.showLog()
+
         print('# logging with in-memory stringio')
         cdo = Cdo(cdfMod=CDF_MOD,logging = True)
         cdo.topo()
@@ -536,52 +642,70 @@ class CdoTest(unittest.TestCase):
         cdo.sinfov(input=cmd)
         cdo.showLog()
 
+    def test_zzzz(self):
+      cdo = Cdo(cdfMod=CDF_MOD)
+      cdo.__print__('test_zzzz')
+      self.assertEqual({},cdo.env)
+
+    def test_AAAA(self):
+      cdo = Cdo(cdfMod=CDF_MOD)
+      cdo.__print__('test_AAAA')
+      self.assertEqual({},cdo.env)
+
+    def test_noOutputOps(self):
+      cdo = Cdo(cdfMod=CDF_MOD)
+      opCount = len(cdo.noOutputOperators)
+      self.assertTrue(opCount > 50)
+      self.assertTrue(opCount < 200)
 
     if MAINTAINERMODE:
         def test_icon_coords(self):
-            cdo = Cdo(cdfMod=CDF_MOD)
-            ifile = DATA_DIR +'/icon/oce_AquaAtlanticBoxACC.nc'
-            ivar  = 't_acc'
-            varIn = cdo.readCdf(ifile)
-            varIn = varIn.variables[ivar]
-            if ('scipy' == cdo.cdfMod ):
-              expected = b'clon clat'
-            else:
-              expected =  u'clon clat'
-            self.assertEqual(expected,varIn.coordinates)
+          cdo = Cdo(cdfMod=CDF_MOD)
+          ifile = DATA_DIR +'/icon/oce_AquaAtlanticBoxACC.nc'
+          ivar  = 't_acc'
+          varIn = cdo.readCdf(ifile)
+          varIn = varIn.variables[ivar]
+          if ('scipy' == cdo.cdfMod ):
+            expected = b'clon clat'
+          else:
+            expected =  u'clon clat'
+          self.assertEqual(expected,varIn.coordinates)
 
-            varOut =cdo.readCdf(cdo.selname(ivar,input=ifile))
-            varOut = varOut.variables[ivar]
-            expected =  u'clat clon'
-            self.assertEqual(expected,varOut.coordinates)
+          varOut =cdo.readCdf(cdo.selname(ivar,input=ifile))
+          varOut = varOut.variables[ivar]
+          expected =  u'clat clon'
+          self.assertEqual(expected,varOut.coordinates)
         def testCall(self):
-            cdo = Cdo(cdfMod=CDF_MOD)
-            print(cdo.sinfov(input=DATA_DIR+'/icon/oce.nc'))
+          cdo = Cdo(cdfMod=CDF_MOD)
+          print(cdo.sinfov(input=DATA_DIR+'/icon/oce.nc'))
         def test_readCdf(self):
-            cdo = Cdo(cdfMod=CDF_MOD)
-            input= "-settunits,days  -setyear,2000 -for,1,4"
-            cdfFile = cdo.copy(options="-f nc",input=input)
-            cdf     = cdo.readCdf(cdfFile)
+          cdo = Cdo(cdfMod=CDF_MOD)
+          input= "-settunits,days  -setyear,2000 -for,1,4"
+          cdfFile = cdo.copy(options="-f nc",input=input)
+          cdf     = cdo.readCdf(cdfFile)
 
-            self.assertEqual(sorted(['lat','lon','for','time']),sorted(list(cdf.variables.keys())))
+          self.assertEqual(sorted(['lat','lon','for','time']),sorted(list(cdf.variables.keys())))
 
 
         def test_phc(self):
-           ifile = DATA_DIR+'/icon/phc.nc'
-           cdo = Cdo(cdfMod=CDF_MOD)
-           cdo.debug = True
-           #cdo.merge(input='/home/ram/data/icon/input/phc3.0/PHC__3.0__TempO__1x1__annual.nc /home/ram/data/icon/input/phc3.0/PHC__3.0__SO__1x1__annual.nc',
-           #          output=ifile,
-           #          options='-O')
-           s = cdo.sellonlatbox(0,30,0,90, input="-chname,SO,s,TempO,t " + ifile,output='my_phc.nc',returnMaArray='s',options='-f nc')
-           plot(np.flipud(s[0,:,:]),ofile='org',title='orgiginal')
-           sfmo = cdo.sellonlatbox(0,30,0,90, input="-fillmiss -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
-           plot(np.flipud(sfmo[0,:,:]),ofile='fm',title='fillmiss')
-           sfm = cdo.sellonlatbox(0,30,0,90, input="-fillmiss2 -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
-           plot(np.flipud(sfm[0,:,:]),ofile='fm2',title='fillmiss2')
-           ssetmisstonn = cdo.sellonlatbox(0,30,0,90, input="-setmisstonn -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
-           plot(np.flipud(ssetmisstonn[0,:,:]),ofile='setmisstonn',title='setmisstonn')
-           #global plot
+          ifile = DATA_DIR+'/icon/phc.nc'
+          cdo = Cdo(cdfMod=CDF_MOD)
+          cdo.debug = DEBUG
+          #cdo.merge(input='/home/ram/data/icon/input/phc3.0/PHC__3.0__TempO__1x1__annual.nc /home/ram/data/icon/input/phc3.0/PHC__3.0__SO__1x1__annual.nc',
+          #          output=ifile,
+          #          options='-O')
+          s = cdo.sellonlatbox(0,30,0,90, input="-chname,SO,s,TempO,t " + ifile,output='test_my_phc.nc',returnMaArray='s',options='-f nc')
+          plot(np.flipud(s[0,:,:]),ofile='org',title='original')
+          sfmo = cdo.sellonlatbox(0,30,0,90, input="-fillmiss -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
+          plot(np.flipud(sfmo[0,:,:]),ofile='fm',title='fillmiss')
+          sfm = cdo.sellonlatbox(0,30,0,90, input="-fillmiss2 -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
+          plot(np.flipud(sfm[0,:,:]),ofile='fm2',title='fillmiss2')
+          ssetmisstonn = cdo.sellonlatbox(0,30,0,90, input="-setmisstonn -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
+          plot(np.flipud(ssetmisstonn[0,:,:]),ofile='setmisstonn',title='setmisstonn')
+          if (parse_version(cdo.version()) >= parse_version('1.7.2')):
+            smooth = cdo.sellonlatbox(0,30,0,90, input="-smooth -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
+            plot(np.flipud(ssetmisstonn[0,:,:]),ofile='smooth',title='smooth')
+          #global plot
           #s_global = cdo.chname('SO,s,TempO,t',input=ifile,output='my_phc.nc',returnMaArray='s',options='-f nc')
           #plot(s_global[0,:,:],ofile='org_global',title='org_global')
           #sfmo_global = cdo.fillmiss(input=" -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
@@ -591,8 +715,84 @@ class CdoTest(unittest.TestCase):
           #ssetmisstonn_global = cdo.setmisstonn(input=" -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
           #plot(ssetmisstonn_global[0,:,:],ofile='setmisstonn_global',title='setmisstonn_global')
 
-            
+        def test_smooth(self):
+          cdo = Cdo(cdfMod='netcdf4')
+          if (parse_version(cdo.version()) >= parse_version('1.7.2')):
+            ifile = DATA_DIR+'/icon/phc.nc'
+            cdo = Cdo(cdfMod=CDF_MOD)
+            cdo.debug = DEBUG
+            #cdo.merge(input='/home/ram/data/icon/input/phc3.0/PHC__3.0__TempO__1x1__annual.nc /home/ram/data/icon/input/phc3.0/PHC__3.0__SO__1x1__annual.nc',
+            #          output=ifile,
+            #          options='-O')
+            smooth = cdo.smooth(input=" -sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
+            plot(np.flipud(smooth[0,:,:]),ofile='smooth',title='smooth')
 
+            smooth2 = cdo.smooth('nsmooth=2',input="-sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
+            plot(np.flipud(smooth2[0,:,:]),ofile='smooth2',title='smooth,nsmooth=2')
+
+            smooth4 = cdo.smooth('nsmooth=4',input="-sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
+            plot(np.flipud(smooth4[0,:,:]),ofile='smooth4',title='smooth,nsmooth=4')
+
+            smooth9 = cdo.smooth9(input="-sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
+            plot(np.flipud(smooth9[0,:,:]),ofile='smooth9',title='smooth9')
+
+            smooth3deg = cdo.smooth('radius=6deg',input="-sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
+            plot(np.flipud(smooth3deg[0,:,:]),ofile='smooth3deg',title='smooth,radius=6deg')
+
+            smooth20 = cdo.smooth('nsmooth=20',input="-sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
+            plot(np.flipud(smooth20[0,:,:]),ofile='smooth20',title='smooth,nsmooth=20')
+
+        def test_xarray_input(self):
+          cdo = Cdo(cdfMod='netcdf4')
+          try:
+            import xarray
+          except:
+            print("no xarray installation available!")
+            return
+
+          dataSet = xarray.open_dataset(cdo.topo('global_0.1',options = '-f nc'))
+
+          print(type(dataSet).__name__)
+
+          dataSet['topo'] = 1.0 + np.abs(dataSet['topo'])
+
+          #check the changes withing xarray
+          self.assertEqual(1.0,np.min(dataSet['topo']))
+
+          xarrayFile = 'test_xarray_topoAbs.nc'
+          dataSet.to_netcdf(xarrayFile)
+
+          #check change via cdo
+          minByCdo = cdo.fldmin(input=xarrayFile,returnArray='topo').min()
+          self.assertEqual(1.0,minByCdo)
+
+          #do the same without explicit tempfile
+          self.assertEqual(1.0,cdo.fldmin(input=dataSet,returnArray='topo').min())
+
+
+        def test_xarray_output(self):
+          cdo = Cdo(cdfMod='netcdf4')
+          try:
+            import xarray
+          except:
+            print("no xarray installation available!")
+            return
+
+          tArray = cdo.topo('global_10.0',options = '-f nc',returnXArray = 'topo')
+          print(tArray)
+
+        def test_xdataset_output(self):
+          cdo = Cdo(cdfMod='netcdf4')
+          try:
+            import xarray
+          except:
+            print("no xarray installation available!")
+            return
+
+          tDataset = cdo.topo('global_10.0',options = '-f nc',returnXDataset = True)
+          print(tDataset)
+
+#===============================================================================
 if __name__ == '__main__':
     suite = unittest.TestLoader().loadTestsFromTestCase(CdoTest)
 #   print(suite)
